/*
 * HttSquash, http server scanner, banner grabber and data retriever
 *
 * Copyright (C) 2009  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "httsquash.h"

/*
 * Write in memory (instead of file).
 * Taken from:
 * http://curl.haxx.se/lxr/source/docs/examples/getinmemory.c
 */
size_t WriteMemoryCallback(void *ptr, size_t size, size_t nmemb, void *data)
{
    size_t realsize = size * nmemb;

    struct MemoryStruct *mem = (struct MemoryStruct *)data;

    mem->memory = (char *) realloc(mem->memory, mem->size + realsize + 1);

    if (mem->memory) {
        memcpy(&(mem->memory[mem->size]), ptr, realsize);
        mem->size += realsize;
        mem->memory[mem->size] = 0;
    }

    return realsize;
}

/*
 * Function called by scanloop()
 * Executed as a child process
 * Gets the ip address and retrieve informations from it
 */
void *scanstep(struct iprange *ipr, int curr)
{
    CURL *curl;
    CURLcode res;
    struct MemoryStruct header, data;

    char addr[MIN_ARG];
    char buff[MAX_DATA];

    // clear structures
    memset(&header, 0, sizeof(header));
    memset(&data, 0, sizeof(data));

    // current address
    if(ipr->ipver==VER_IPV4)
        sprintf(addr, "%s.%u", ipr->block, curr);
    else
        sprintf(addr, "%s:%x", ipr->block, curr);

    // initialize curl
    curl = curl_easy_init();
    if (curl == NULL) {
        fprintf(stderr, "Error: initialize libcurl\n");
        exit(1);
    }

    // debug
    //curl_easy_setopt(curl, CURLOPT_VERBOSE, 1);

    // set port and timeout
    curl_easy_setopt(curl, CURLOPT_PORT, ipr->port);
    curl_easy_setopt(curl, CURLOPT_TIMEOUT, ipr->time);


    // set request type
    curl_easy_setopt(curl, CURLOPT_CUSTOMREQUEST, ipr->type);

    // don't verify SSL certificate
    curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0);
    curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0);

    // set request url
    snprintf(buff, MAX_DATA-1, "%s%s%s", ipr->https ? "https://" : "http://", addr, ipr->url);
    curl_easy_setopt(curl, CURLOPT_URL, buff);


    // set personalized headers
    if(ipr->headers) {
        curl_easy_setopt(curl, CURLOPT_HTTPHEADER, ipr->headers);
    }


    // set memory write callback
    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
    curl_easy_setopt(curl, CURLOPT_WRITEHEADER, (void *)&header);
    curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void *)&data);

    // get result
    res = curl_easy_perform(curl);
    if(res!=0) {
        curl_easy_cleanup(curl);
        exit(1);
    }

    // *** Print All ***
    printf("FOUND: %s %d\n%s%s%s%s%s",

            // header
            addr,
            ipr->port,
            header.memory,

            // data
            ipr->body ? "DATA:\n" : "",
            ipr->body ? data.memory : "",
            ipr->body ? "\n\n" : "",

            // cookie jar
            ipr->cookiejar ? "%%\n\n" : "");

    // free structures
    if(header.memory) free(header.memory);
    if(data.memory) free(data.memory);

    // cleanup
    curl_easy_cleanup(curl);

    // exit
    exit(0);
}

/*
 * Scan loop that assembles current ip address
 * and exec a callback function
 */
void scanloop(struct iprange *ipr, int max_pr)
{
    int net_end, curr, curr2;
    int pr_count=0;
    int status;

    curr = ipr->start;

    // no extended range
    if(ipr->range_ext==0) {
        while(curr <= ipr->end) {

            // child process
            if (fork() == 0) {
                scanstep(ipr, curr);
                exit(0);
            }

            // increment counters
            curr++;
            pr_count++;

            // check the number of processes
            if(pr_count >= ipr->max_childs) {

                // wait for the other childs
                for(; pr_count; pr_count--) {
                    wait(&status);
                }
            }

        }
    }

    // extended range (only ipv4)
    else {
        curr2 = ipr->start2;

        while(curr2 <= ipr->end2) {

            // set net_end
            if(curr2==ipr->end2) {
                net_end=ipr->end;
            }
            else {
                net_end=254;
            }

            while(curr <= net_end) {

                // child process
                if (fork() == 0) {
                    scanstep(ipr, curr);
                    exit(0);
                }

                // increment counters
                curr++;
                pr_count++;

                // check the number of processes
                if(pr_count >= ipr->max_childs) {

                    // wait for the other childs
                    for(; pr_count; pr_count--) {
                        wait(&status);
                    }
                }

            }

            curr = 0;
            curr2++;
        }
    }

    // check the number of processes
    if(pr_count) {

        // wait for the other childs
        for(; pr_count; pr_count--) {
            wait(&status);
        }
    }

    return;
}

/*
 * Initialize the program
 */
struct iprange *prog_initialize()
{
    // alloc ipr
    struct iprange *ipr = (struct iprange *) malloc(sizeof(struct iprange));
    memset(ipr, 0, sizeof(struct iprange));

    return ipr;
}

/*
 * Cleanup the program
 */
void prog_cleanup(struct iprange *ipr)
{
    // free header list
    if(ipr->headers) curl_slist_free_all(ipr->headers);

    // dealloc ipr
    free(ipr);
}

/*
 * Main function
 */
int main(int argc, char **argv)
{
    struct iprange *ipr = NULL;

    // Initialize the program
    ipr = prog_initialize();

    // Parse options
    parseopt(ipr, argc, argv);

    // Init scan loop
    scanloop(ipr, ipr->max_childs);

    // Cleanup the program
    prog_cleanup(ipr);

    exit(0);
}

