/*
 * ARGPARSER.H
 * Some utilities and parsers...
 *
 * Copyright (C) 2009  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __ARGPARSER_H__
#define __ARGPARSER_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <getopt.h>
#include <regex.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <time.h>

#include <string>

#define VERSION "v0.7.7.2"
#define AUTHOR "Acri Emanuele (crossbower@gmail.com)"

#define MIN_ARG 128
#define MAX_ARG 2048

#define IPTABLES_ENABLE  "iptables -A OUTPUT -p tcp --tcp-flags ALL RST -j DROP"
#define IPTABLES_DISABLE "iptables -D OUTPUT -p tcp --tcp-flags ALL RST -j DROP"

#define BLACKHOLE_ENABLE  "sysctl net.inet.tcp.blackhole=2"
#define BLACKHOLE_DISABLE "sysctl net.inet.tcp.blackhole=0"

/*
 * Options structure
 */
struct options {

    int dport;    // destination port
    int xport;    // first source port
    int yport;    // last source port
    int time;     // sleep time
    int alarm;    // alarm time
    int window;   // tcp window size
    int frag;     // fragment offset
    int fragc;    // fragment counter

    int ipver;    // ip version

    int syn:1;    // only syn flood
    int ack:1;    // send acknowledgment packets
    int fin:1;    // send finalize packets
    int rst:1;    // send reset packets
    int mlt:1;    // multistage payload
    int loop:1;   // infinite loop

    int verbose;  // verbosity

    char *dstaddr;            // destination address
    in_addr_t dstip;          // destination ip
    
    char *srcaddr;            // source address
    in_addr_t srcip;          // source ip

    char *dstmac;             // destination mac
    char *srcmac;             // source mac

    char *device;             // network interface

    const char *firewall[2];  // firewall rules

    char payload[MAX_ARG*4];  // payload
    char *mlt_payload;        // multistage payload

} progopt;
#define VER_IPV4 4
#define VER_IPV6 6

/*
 * Program usage
 */
const char *usage_text =
    "\nLetDown 3wh+payload flooder " VERSION " - " AUTHOR "\n"
    "Usage:\n"
    "  letdown -d dst_ip -p dst_port -D dst_mac  [options]\n"
    "Options:\n"
    "  -d\tdestination ip address, target\n"
    "  -D\tdestination mac address or router mac address\n"
    "  -p\tdestination port\n"
    "  -s\tsource ip address\n"
    "  -S\tsource mac address\n"
    "  -x\tfirst source port (default 1025)\n"
    "  -y\tlast source port (default 65534)\n"
    "  -l\tenables infinite loop mode\n"
    "  -i\tnetwork interface\n"
    "  -t\tsleep time in microseconds (default 10000)\n"
    "  -a\tmax time in second for waiting responses\n"
    "Extra options:\n"
    "  -v\tverbosity level (0=quiet, 1=normal, 2=verbose)\n"
    "  -f\tautomagically set firewall rules for blocking\n"
    "    \trst packet generated by the kernel\n"
    "    \texamples: -f iptables, -f blackhole (for freebsd)\n"
    "  -L\tspecial interaction levels with the target\n"
    "    \ts  syn flooding, no 3-way-handshake\n"
    "    \ta  send acknowledgment packets (polite mode)\n"
    "    \tf  send finalize packets (include polite mode)\n"
    "    \tr  send reset packets (check firewall rules...)\n"
    "  -W\twindow size for ack packets (ex: 0-window attack)\n"
    "  -O\tenable ack fragmentation and set fragment offset delta\n"
    "  -C\tfragment counter if fragmentation is enabled (default 1)\n"
    "  -P\tpayload file (see payloads directory...)\n"
    "  -M\tmultistage payload file (see payloads directory...)\n";

/*
 * Regular expression used by
 * parseopt() function
 */
const char *reg_ipv4_range =
    "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}[-][0-9]{1,3}$";

const char *reg_ipv4_addr =
    "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}$";

const char *reg_ipv6_range =
    "^[0-9a-fA-F:]{1,42}[-][0-9a-fA-F]{1,4}$";

const char *reg_ipv6_addr =
    "^[0-9a-fA-F:]{1,42}$";

const char *reg_mac =
    "^[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}:[0-9a-fA-F]{2}$";

const char *reg_number =
    "\\d+?";

/*
 * Print the error on stderr and exit
 */
void p_error(const char *err)
{
    fprintf(stderr, err);
    exit(EXIT_FAILURE);
}

/*
 * Program usage
 * First pararameter can be stdout or stderr
 * Second parameter is the error message or NULL
 */
void usage(FILE *out, const char *error)
{
    fputs(usage_text, out);

    if(error)
        fputs(error, out);

    exit(1);
}

/*
 * Regular expression matching function
 * Returns:
 *    -1  error on pattern compilation
 *     0  don't match
 *     1  match
 */
int regmatch(const char *str, const char *regex)
{
    regex_t re;

    // Compile the regex pattern.
    if (regcomp(&re, regex, REG_EXTENDED | REG_NOSUB) != 0) {
        fprintf(stderr, "Error on regex pattern compilation.\n");
        return -1;
    }

    // Use the pattern on the passed string
    if (regexec(&re, str, 0, NULL, 0))  {
        return 0;
    }

    regfree(&re);
    return 1;
}

/*
 * Check verbosity
 */
int verbosity(int verbose)
{
    if(verbose > progopt.verbose)
        return 0;
    return 1;
}

/*
 * Print time
 */
void printtime()
{
  struct tm *local;
  time_t t;

  t = time(NULL);
  local = localtime(&t);
  printf("%s", asctime(local));
}

/*
 * Get simple payload from file
 */
int getpayload(char *path) {
    FILE *file;

    // open file
    if((file=fopen(path, "r"))==NULL)
        p_error("\nError opening payload file.\n");

    if(!fread(progopt.payload, 1, MAX_ARG*4-1, file))
        p_error("\nError reading  payload from file.\n");

    fclose(file);

    return 1;
}

/*
 * Parser for command line options
 * See getopt(3)...
 */
int parseopt(int argc, char **argv)
{
    char c;

    // Clean

    memset(&progopt, 0, sizeof(progopt));

    // Command line options and default values

    progopt.dport = 0;     // destination port
    progopt.xport = 1025;  // first source port
    progopt.yport = 65534; // last source port
    progopt.time  = 10000; // sleep time
    progopt.alarm = 0;     // alarm time
    progopt.window = 5840; // tcp window size
    progopt.frag = 0;      // fragment offset
    progopt.fragc = 1;     // fragment counter
    progopt.syn = 0;       // syn flooding only
    progopt.ack = 0;       // send acknowledgment packets
    progopt.fin = 0;       // send finalize packets
    progopt.rst = 0;       // send reset packets
    progopt.mlt = 0;       // multistage payload
    progopt.loop = 0;      // infinite loop
    progopt.ipver = 0;     // ip version
    progopt.verbose = 1;   // verbosity

    char *firewall = NULL;
    char *payload = NULL;

    // short options for getopt()
    char shortopt[] = "d:s:D:S:p:x:y:li:t:a:f:v:L:W:O:C:P:M:";

    // getopt() loop
    while ((c = getopt (argc, argv, shortopt)) != -1)
        switch (c) {

            case 'd': // Destination

                progopt.dstip = inet_addr(optarg);

                if(progopt.dstip) {
                    progopt.ipver=VER_IPV4;
                    progopt.dstaddr = optarg;
                }

                else {
                    usage(stderr, "\nArgument -d is invalid.\n");
                }

                break;

            case 's': // Source address

                progopt.srcip = inet_addr(optarg);
                
                if(progopt.srcip) {
                    progopt.srcaddr = optarg;
                }

                else {
                    usage(stderr, "\nArgument -s is invalid.\n");
                }

                break;

            case 'D': // Destination mac address

                if(!regmatch(optarg, reg_mac))
                    usage(stderr, "\nArgument -D is invalid.\n");

                progopt.dstmac = optarg;

                break;

            case 'S': // Source mac address

                if(!regmatch(optarg, reg_mac))
                    usage(stderr, "\nArgument -S is invalid.\n");

                progopt.srcmac = optarg;

                break;

            case 'p': // Port

                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument -p is invalid.\n");

                progopt.dport = atoi(optarg);

                // check value
                if(progopt.dport < 1 || progopt.dport > 65355)
                     usage(stderr, "\nArgument -p is invalid.\n");

                break;

            case 'x': // First source port

                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument -x is invalid.\n");

                progopt.xport = atoi(optarg);

                // check value
                if(progopt.xport < 1 || progopt.xport > 65355)
                     usage(stderr, "\nArgument -x is invalid.\n");

                break;

            case 'y': // Last source port

                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument -y is invalid.\n");

                progopt.yport = atoi(optarg);

                // check value
                if(progopt.yport < 1 || progopt.yport > 65355)
                    usage(stderr, "\nArgument -y is invalid.\n");

                break;

            case 'l': // Infinite loop
                progopt.loop = 1;
                break;

            case 'i': // Interface
                progopt.device = optarg;
                break;

            case 't': // Sleep time

                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument -t is invalid.\n");

                progopt.time = atoi(optarg);

                break;

            case 'a': // Alarm time

                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument -a is invalid.\n");

                progopt.alarm = atoi(optarg);

                break;

            case 'f': // Firewall rule
                firewall = optarg;
                break;

            case 'v': // Verbosity level

                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument -v is invalid.\n");

                progopt.verbose = atoi(optarg);

                break;

            case 'L': // Level of interaction

                // rst (include ack)
                if(strstr(optarg, "r")) {
                    progopt.ack = 1; progopt.rst = 1;
                }

                // fin (include ack)
                else if(strstr(optarg, "f")) {
                    progopt.ack = 1; progopt.fin = 1;
                }

                // ack (polite mode)
                else if(strstr(optarg, "a")) {
                    progopt.ack = 1;
                }

                // syn flood
                else if(strstr(optarg, "s")) {
                    progopt.syn = 1;
                }

                else {
                    usage(stderr, "\nArgument -L is invalid.\n");
                }

                break;

            case 'W': // Window size

                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument -W is invalid.\n");

                progopt.window = atoi(optarg);

                break;

            case 'O': // Fragment offset

                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument -O is invalid.\n");

                progopt.frag = atoi(optarg);

                break;

            case 'C': // Fragment counter

                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument -C is invalid.\n");

                progopt.fragc = atoi(optarg);

                break;

            case 'P': // Payload
                payload = optarg;
                break;

            case 'M': // Multistage payload
                progopt.mlt_payload = optarg;
                progopt.mlt = 1;
                break;

            case '?':
            default:
                usage(stderr, NULL);
        }

    //check collected args and fill options struct
    if(!progopt.dstaddr || !progopt.dport || !progopt.dstmac)
        usage(stdout, NULL);

    if(progopt.xport > progopt.yport)
        usage(stderr, "\nAre you ok? Check -x and -y please ;)\n");

    if(progopt.verbose < 0)
        progopt.verbose = 0;

    if(firewall) {
        if(!strcmp("iptables", firewall)) {
            progopt.firewall[0] = IPTABLES_ENABLE;
            progopt.firewall[1] = IPTABLES_DISABLE;
        }
        else if(!strcmp("blackhole", firewall)) {
            progopt.firewall[0] = BLACKHOLE_ENABLE;
            progopt.firewall[1] = BLACKHOLE_DISABLE;
        }
        else
           usage(stderr, "\nI don't know this firewall\n");
    }

    if(payload)
        getpayload(payload);

    if(progopt.mlt_payload) {
        progopt.ack = 1; // required by multistage payloads
    }

    return 1;
}

#endif /* __ARGPARSER_H__ */
