#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <ifaddrs.h>
#include <netdb.h>
#include <string.h>

#ifndef linux
#include <net/if_dl.h>
#else
#include <sys/ioctl.h>
#include <net/if.h>
#endif

/*
 * Get the MAC address of the specified interface
 *
 * Return:
 *   1 = success
 *   0 = failure
 */
int if_mac_address(char *interface, char *if_mac)
{
    struct ifaddrs *ifaddr = NULL;
    struct ifaddrs *ifa = NULL;
    int found = 0;

    if (getifaddrs(&ifaddr) != 0) {
        return 0;
    }

    /*
     * Walk through linked list, maintaining head pointer so we
     * can free list later
     */
    
    for (ifa=ifaddr; ifa && !found; ifa=ifa->ifa_next) {

        if (
#ifndef linux
                    ifa->ifa_addr->sa_family == AF_LINK // bsd
#else
                    ifa->ifa_addr->sa_family == AF_INET // linux
#endif
            )
        {
            if (strcmp(ifa->ifa_name, interface) == 0) {

                found = 1;

#ifndef linux

                // the bsd way
                struct sockaddr_dl *sdl = (struct sockaddr_dl *)ifa->ifa_addr;
                memcpy(if_mac, LLADDR(sdl), 6);

#else

                // the linux way
                int fd;
                struct ifreq ifr;

                fd = socket(AF_INET, SOCK_DGRAM, 0);

                ifr.ifr_addr.sa_family = AF_INET;
                strncpy(ifr.ifr_name, interface, IFNAMSIZ-1);

                ioctl(fd, SIOCGIFHWADDR, &ifr);

                close(fd);

                memcpy(if_mac, ifr.ifr_hwaddr.sa_data, 6);

#endif

            }
        }
    }

    freeifaddrs(ifaddr);

    return found;
}

/*
 * Get the IP address of the specified interface
 *
 * Return:
 *   1 = success
 *   0 = failure
 */
int if_ip_address(char *interface, char *if_ip)
{
    struct ifaddrs *ifaddr, *ifa;
    int err, found = 0;

    if (getifaddrs(&ifaddr) == -1) {
        return 0;
    }

    /*
     * Walk through linked list, maintaining head pointer so we
     * can free list later
     */

    for (ifa = ifaddr; ifa && !found; ifa = ifa->ifa_next) {
        if (ifa->ifa_addr->sa_family == AF_INET) {
            if (strcmp(ifa->ifa_name, interface) == 0) {

                found = 1;

                /* get ip address */

                err = getnameinfo(ifa->ifa_addr, sizeof(struct sockaddr_in), if_ip, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
                if (err != 0) {
                    return 0;
                }

            }
        }
    }

    freeifaddrs(ifaddr);

    return found;
}

#if 0
int main(int argc, char **argv) {

    unsigned char if_mac[6];
    char if_ip[NI_MAXHOST];

    if(argc<2) {
       printf("usage: %s <interface>\n", argv[0]);
       return 1; 
    }

    // get ip
    if(!if_ip_address(argv[1], if_ip)) {
        fprintf(stderr, "Error getting the IP address.\n");
        return 1;
    }

    // get mac
    if(!if_mac_address(argv[1], if_mac)) {
        fprintf(stderr, "Error getting the MAC address.\n");
        return 1;
    }

    // print all
    printf("%s %s %02x:%02x:%02x:%02x:%02x:%02x\n", argv[1], if_ip,
            if_mac[0], if_mac[1], if_mac[2],
            if_mac[3], if_mac[4], if_mac[5]);

    return 0;

}
#endif


