/*
 * ReverseRaider, domain scanner.
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 * 
 * From an idea of Jeremy Brown --- http://jbrownsec.blogspot.com/
 *
 * Thanks to:
 * Martin Holst Swende <martin.holst_swende@msc.se>
 * for wordlistscanner() function
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "reverseraider.h"

/*
 * Callback function
 */
static void callback(void *arg, int status, int timeouts, struct hostent *host)
{
    char **p;

    if(status != ARES_SUCCESS) return;

    for (p = host->h_addr_list; *p; p++) {
        
        char addr_buf[46] = "??";

        inet_ntop(host->h_addrtype, *p, addr_buf, sizeof(addr_buf));
        
        printf("%-32s\t%s\n", host->h_name, addr_buf);

        if (host->h_aliases[0]) {
            int i;

            for (i = 0; host->h_aliases[i]; i++) {
                printf("%-32s\t%s\n", host->h_aliases[i], addr_buf);
            }
        }
    }
}

/*
 * Prepare ares options and mask for ares_init_options()
 */
void prep_ares_opt(struct options *opt, struct ares_options *options, int *optmask)
{
    static struct in_addr ia;
    int status=0;
    
    *optmask = 0;

    // check for personalized nameserver
    if(opt->nameserver[0]!='\0') {
        if(!inet_pton(AF_INET, opt->nameserver, &ia)) {
            fprintf(stderr, "ares_init: %s\n", ares_strerror(status));
            exit(1);
        }
        options->servers = &ia;
        options->nservers=1;
        *optmask |= ARES_OPT_SERVERS;
    }
    
    // check for personalized time
    if(opt->timeout) {
        options->timeout = opt->timeout;
        *optmask |= ARES_OPT_TIMEOUT;
    }
    
    // TCP queries
    if(opt->usetcp) {
        *optmask |= ARES_FLAG_USEVC;
    }
    
    // no recursion
    if(opt->norecurse) {
        *optmask |= ARES_FLAG_NORECURSE;
    }
}

/*
 * Brute force wordlist scanner
 */
void wordlistscanner(struct options *opt)
{
    char host[MAX_ARG];
    char prefix[MAX_ARG];
    
    ares_channel channel;
    struct ares_options options;
    int optmask = 0;
    
    FILE *wordlist;
    
    int i=0, status, nfds;
    fd_set read_fds, write_fds;
    struct timeval *tvp, tv;
    
    int permcount = 40; // permutation counter
    
    // open wordlist  
    wordlist = fopen(opt->wordlist, "r");
    if(!wordlist) {
        fprintf(stderr, "Error opening %s\n", opt->wordlist);
        exit(0);
    }
    
    // prepare ares options
    prep_ares_opt(opt, &options, &optmask);
        
    // open ares channel
    status = ares_init_options(&channel, &options, optmask);
    if (status != ARES_SUCCESS) {
        fprintf(stderr, "ares_init: %s\n", ares_strerror(status));
        exit(1);
    }
    
    // scan loop
    fflush(stdout);
    while(!feof(wordlist)) {
    
        // get prefix
        fgets(prefix, MAX_ARG, wordlist);
        
        // delete newlines
        for(i=0; prefix[i]!=0; i++)
            if (prefix[i]=='\n') {
            prefix[i]='\0';
            break;
        }
    
        memset(host, MAX_ARG, 0);
        snprintf(host, MAX_ARG, "%s.%s", prefix, opt->domain);
        
        // query
        ares_gethostbyname(channel, host, AF_INET, callback, host);
        ares_gethostbyname(channel, host, AF_INET6, callback, host);
        
        // Permutation?
        while(opt->permutation && permcount) {
            
            // Permutation: prefix-number-domain
            for(i=0; i<10; i++) {
                
                // permutation type
                if (permcount > 30)
                    snprintf(host, MAX_ARG, "%s%d.%s", prefix, i, opt->domain);
                else if (permcount > 20)
                    snprintf(host, MAX_ARG, "%s0%d.%s", prefix, i, opt->domain);
                else if (permcount > 10)
                    snprintf(host, MAX_ARG, "%d%s.%s", i, prefix, opt->domain);
                else if (permcount > 0)
                    snprintf(host, MAX_ARG, "0%d%s.%s", i, prefix, opt->domain);
        
                // query
                ares_gethostbyname(channel, host, AF_INET, callback, host);
                ares_gethostbyname(channel, host, AF_INET6, callback, host);
                
                fflush(stdout);
                
                permcount--;
            }
                        
        } // permutation end
    }
            
    // wait for all queries to complete
    while (1) {
        FD_ZERO(&read_fds);
        FD_ZERO(&write_fds);
        nfds = ares_fds(channel, &read_fds, &write_fds);
        
        if (nfds == 0) break;
        
        tvp = ares_timeout(channel, NULL, &tv);
        select(nfds, &read_fds, &write_fds, NULL, tvp);
        ares_process(channel, &read_fds, &write_fds);
    }

    ares_destroy(channel);
    fclose(wordlist);
}

/*
 * Reverse resolution scanner
 */
void reversescanner(struct options *opt)
{
    char addr[MAX_ARG];

    ares_channel channel;
    struct ares_options options;
    int optmask = 0;

    struct sockaddr_in sa;
    struct sockaddr_in6 sa6;

    int status, nfds;
    fd_set read_fds, write_fds;
    struct timeval *tvp, tv;

    int curr;

    curr = opt->start;
    
    // prepare ares options
    prep_ares_opt(opt, &options, &optmask);

    // open ares channel
    status = ares_init_options(&channel, &options, optmask);
    if (status != ARES_SUCCESS) {
        fprintf(stderr, "ares_init: %s\n", ares_strerror(status));
        exit(1);
    }
    
    // scan loop
    while(curr <= opt->end) {

        // current address
        if(opt->ipver==VER_IPV4)
            sprintf(addr, "%s.%u", opt->block, curr);
        else
            sprintf(addr, "%s:%x", opt->block, curr);
        
        // fill sockaddr struct
        if(opt->ipver==VER_IPV4) {
            memset(&sa, 0, sizeof(sa));
            
            sa.sin_family = AF_INET;
            if(inet_pton(AF_INET, addr, &(sa.sin_addr)) < 0) {
                fprintf(stderr, "Address %s is not valid\n", addr);
                exit(0);
            }
        }
        else {
            memset(&sa6, 0, sizeof(sa6));
            
            sa6.sin6_family = AF_INET6;
            
            if(inet_pton(AF_INET6, addr, &(sa6.sin6_addr)) < 0) {
                fprintf(stderr, "Address %s is not valid\n", addr);
                exit(0);
            }
        }
        
        // query
        if(opt->ipver==VER_IPV4) {
            ares_gethostbyaddr(channel, &(sa.sin_addr), sizeof(sa.sin_addr), AF_INET, callback, addr);
        }
        else {
            ares_gethostbyaddr(channel, &(sa6.sin6_addr), sizeof(sa6.sin6_addr), AF_INET6, callback, addr);
        }
        
        curr++;
    }
    
    // wait for all queries to complete
    while (1) {
        FD_ZERO(&read_fds);
        FD_ZERO(&write_fds);
        nfds = ares_fds(channel, &read_fds, &write_fds);
        
        if (nfds == 0) break;
        
        tvp = ares_timeout(channel, NULL, &tv);
        select(nfds, &read_fds, &write_fds, NULL, tvp);
        ares_process(channel, &read_fds, &write_fds);
    }

    ares_destroy(channel);
}

/*
 * Reverse resolution scanner using ip list file
 */
void reversescannerfile(struct options *opt)
{
    char addr[MIN_ARG];
    int ipver = VER_IPV4;
    
    ares_channel channel;
    struct ares_options options;
    int optmask = 0;
    
    FILE *iplist;
    
    struct sockaddr_in sa;
    struct sockaddr_in6 sa6;
    
    int i=0, status, nfds;
    fd_set read_fds, write_fds;
    struct timeval *tvp, tv;
    
    // open ip list file 
    iplist = fopen(opt->iplist, "r");
    if(!iplist) {
        fprintf(stderr, "Error opening %s\n", opt->iplist);
        exit(0);
    }
    
    // prepare ares options
    prep_ares_opt(opt, &options, &optmask);
        
    // open ares channel
    status = ares_init_options(&channel, &options, optmask);
    if (status != ARES_SUCCESS) {
        fprintf(stderr, "ares_init: %s\n", ares_strerror(status));
        exit(1);
    }
    
    // scan loop
    fflush(stdout);
    while(!feof(iplist)) {
    
        // get ip address
        fgets(addr, MIN_ARG-1, iplist);
        
        // delete newlines
        for(i=0; i<MIN_ARG; i++) {
            if (addr[i]=='\n') {
                addr[i]='\0';
                break;
            }
        }
        
        // check address type
        if(regmatch(addr, reg_ipv4_addr)) {
            ipver=VER_IPV4;
        } else {
            ipver=VER_IPV6;
        }
        
        // fill sockaddr struct
        if(ipver==VER_IPV4) {
            memset(&sa, 0, sizeof(sa));
            
            sa.sin_family = AF_INET;
            if(inet_pton(AF_INET, addr, &(sa.sin_addr)) < 0) {
                fprintf(stderr, "Address %s is not valid\n", addr);
                continue;
            }
        }
        else {
            memset(&sa6, 0, sizeof(sa6));
            
            sa6.sin6_family = AF_INET6;
            
            if(inet_pton(AF_INET6, addr, &(sa6.sin6_addr)) < 0) {
                fprintf(stderr, "Address %s is not valid\n", addr);
                continue;
            }
        }
        
        // query
        if(ipver==VER_IPV4) {
            ares_gethostbyaddr(channel, &(sa.sin_addr), sizeof(sa.sin_addr), AF_INET, callback, addr);
        }
        else {
            ares_gethostbyaddr(channel, &(sa6.sin6_addr), sizeof(sa6.sin6_addr), AF_INET6, callback, addr);
        }
    
    }
            
    // wait for all queries to complete
    while (1) {
        FD_ZERO(&read_fds);
        FD_ZERO(&write_fds);
        nfds = ares_fds(channel, &read_fds, &write_fds);
        
        if (nfds == 0) break;
        
        tvp = ares_timeout(channel, NULL, &tv);
        select(nfds, &read_fds, &write_fds, NULL, tvp);
        ares_process(channel, &read_fds, &write_fds);
    }

    ares_destroy(channel);
    fclose(iplist);
}

/*
 * Program init
 */
int main(int argc, char **argv)
{

    struct options opt;

    // parse options
    parseopt(&opt, argc, argv);

    // Wordlist scanning
    if(opt.domain[0]!='\0') {
        wordlistscanner(&opt);
    }

    else if (opt.iplist[0]!='\0') {
        reversescannerfile(&opt);
    }
       
    // Reverse scanning
    else {
        reversescanner(&opt);
    }
    
    return 0;
}

